﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// ODA
using Teigha.Runtime;
using Teigha.DatabaseServices;
using Teigha.Geometry;
using Teigha.BoundaryRepresentation;
using Teigha.Colors;

// Bricsys
using Bricscad.ApplicationServices;
using Bricscad.Runtime;
using Bricscad.EditorInput;


// alias
using AcAp = Bricscad.ApplicationServices;
using AcDb = Teigha.DatabaseServices;
using AcGe = Teigha.Geometry;
using AcEd = Bricscad.EditorInput;

/*
 BRX Documentation 

        UX_EXPORT_FUNCTION Acad::ErrorStatus AcDb3dSolid::createSculptedSolid(const AcArray< AcDbEntity * > &  	limitingBodies,
		                                                                      const AcGeIntArray &  	limitingFlags ) 		 
        Creates a 3d solid from the enclosed volume formed by the limiting bodies.
        Parameters:
            [in] 	limitingBodies 	surfaces and solids to form the enclosed space
            [in] 	limitingFlags 	flags (not currently used)
        Returns:
            Acad::eOk if successful, an errorcode otherwise 
 */

namespace SurfSculptX
{
    public class CreateSculptedSol
    {
        [CommandMethod("SURFSCULPTX", CommandFlags.UsePickSet)]
        public void createSculpted()
        {
            // get the usuals
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            Transaction tr = db.TransactionManager.StartTransaction();

            // select filtered enclosing solids and surfaces.... 
            PromptSelectionOptions opts = new PromptSelectionOptions();
            opts.MessageForAdding = "Select enclosing Surfaces and Solids ";

            TypedValue[] selFilter = new TypedValue[10] { new TypedValue((int)DxfCode.Operator, "<OR"),
                                                         new TypedValue(0 , "SURFACE"),
                                                         new TypedValue(0 , "SWEPTSURFACE"),
                                                         new TypedValue(0 , "LOFTEDSURFACE"),
                                                         new TypedValue(0 , "PLANESURFACE"),
                                                         new TypedValue(0 , "REVOLVEDSURFACE"),
                                                         new TypedValue(0 , "EXTRUDEDSURFACE"),
                                                         new TypedValue(0 , "NURBSURFACE"),
                                                         //new TypedValue(0 , "MESH"),
                                                         new TypedValue(0 , "3DSOLID"),
                                                         new TypedValue((int)DxfCode.Operator, "OR>")
                                                        };

            PromptSelectionResult res = ed.GetSelection(opts, new SelectionFilter(selFilter));

            // if selection ok
            if (res.Status == PromptStatus.OK)
            {
                SelectionSet ss = res.Value;
                ObjectId[] idarray = ss.GetObjectIds();

                Solid3d pSolid = new Solid3d();
                Entity SurfSolEnt;
                List<Entity> entList = new List<Entity>();
                // this parameter according to BRX documentation above is not currently used !
                IntegerCollection limits = new IntegerCollection(idarray.Length);

                BlockTable bt = (BlockTable)tr.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord ms = (BlockTableRecord)tr.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);

                using (tr)
                {
                    try
                    {
                        for (int i = 0; i < idarray.Length; i++)
                        {
                            // open the 1st entity (surface or solid)...
                            SurfSolEnt = tr.GetObject(idarray[i], OpenMode.ForRead) as Entity;
                            // add it to an entity list...
                            entList.Add((Entity)SurfSolEnt.Clone());
                        }

                        // create sculpted solids
                        pSolid.CreateSculptedSolid(entList.ToArray(), limits);

                        pSolid.SetDatabaseDefaults(db);
                        ms.AppendEntity(pSolid);
                        tr.AddNewlyCreatedDBObject(pSolid, true);
                        //set the current color
                        pSolid.Color = db.Cecolor;
                       
                        tr.Commit();

                        //Dispose all Clones...
                        foreach (Entity clone in entList)
                        {
                            clone.Erase();
                            clone.Dispose();
                        }

                    }
                    catch
                    {
                    }
                    finally
                    {
                    }

                }
            }
        }

    }
}
